@@ -37,6 +37,7 @@ read_cohort_set <- function(path) {
     rlang::abort(glue::glue("The directory {path} does not exist!"))
   }
 
+
   if (file.exists(file.path(path, "CohortsToCreate.csv"))) {
     cohortsToCreate <- readr::read_csv(file.path(path, "CohortsToCreate.csv"), show_col_types = FALSE) %>%
       dplyr::mutate(jsonPath = file.path(path, .data$jsonPath)) %>%
@@ -53,7 +54,11 @@ read_cohort_set <- function(path) {
       dplyr::mutate(json = purrr::map(.data$json_path, readr::read_file))
   }
 
-  cohortsToCreate <- dplyr::select(cohortsToCreate, "cohort_definition_id", "cohort_name", "cohort", "json")
+  # snakecase name can be used for column names or filenames
+  cohortsToCreate <- cohortsToCreate %>%
+    dplyr::mutate(cohort_name_snakecase = snakecase::to_snake_case(.data$cohort_name)) %>%
+    dplyr::select("cohort_definition_id", "cohort_name", "cohort", "json", "cohort_name_snakecase")
+
   class(cohortsToCreate) <- c("CohortSet", class(cohortsToCreate))
   return(cohortsToCreate)
 }
@@ -101,7 +106,7 @@ readCohortSet <- read_cohort_set
 #' or a named list of Capr cohort definitions.
 #' @param compute_attrition,computeAttrition Should attrition be computed? TRUE (default) or FALSE
 #' @param overwrite Should the cohort table be overwritten if it already
-#' exists? TRUE or FALSE (default)
+#' exists? TRUE (default) or FALSE
 #' @export
 #' @examples
 #' \dontrun{
@@ -122,42 +127,13 @@ readCohortSet <- read_cohort_set
 #' }
 generateCohortSet <- function(cdm,
                               cohortSet,
-                              name = "cohort",
+                              name,
                               computeAttrition = TRUE,
-                              overwrite = FALSE) {
+                              overwrite = TRUE) {
 
   rlang::check_installed("CirceR")
   rlang::check_installed("SqlRender")
 
-  checkmate::assertClass(cdm, "cdm_reference")
-  con <- attr(cdm, "dbcon")
-  checkmate::assertTRUE(DBI::dbIsValid(con))
-
-  withr::local_options(list("cli.progress_show_after" = 0))
-  # cli::cli_progress_bar(
-    # total = nrow(cohortSet),
-    # format = "Generating cohorts {cli::pb_bar} {cli::pb_current}/{cli::pb_total}")
-  # cli::cli_progress_update(set = 0)
-
-  assert_write_schema(cdm) # required for now
-
-  checkmate::assertLogical(computeAttrition, len = 1)
-  checkmate::assertLogical(overwrite, len = 1)
-
-  write_schema <- attr(cdm, "write_schema")
-  checkmate::assert_character(write_schema,
-                              min.chars = 1,
-                              min.len = 1,
-                              max.len = 3,
-                              null.ok = FALSE)
-
-  if ("prefix" %in% names(write_schema)) {
-    prefix <- unname(write_schema["prefix"])
-    # write_schema <- write_schema[-which(names(write_schema) == "prefix")]
-  } else {
-    prefix <- ""
-  }
-
   if (!is.data.frame(cohortSet)) {
     if (!is.list(cohortSet)) {
       rlang::abort("cohortSet must be a dataframe or a named list of Capr cohort definitions")
@@ -180,6 +156,35 @@ generateCohortSet <- function(cdm,
 
   checkmate::assertDataFrame(cohortSet, min.rows = 1, col.names = "named")
 
+  cli::cli_alert_info("Generating {nrow(cohortSet)} cohort{?s}")
+  withr::local_options(list("cli.progress_show_after" = 0, "cli.progress_clear" = FALSE))
+
+
+  checkmate::assertClass(cdm, "cdm_reference")
+  con <- attr(cdm, "dbcon")
+  checkmate::assertTRUE(DBI::dbIsValid(con))
+  checkmate::assert_character(name, len = 1, min.chars = 1, any.missing = FALSE, pattern = "[a-zA-Z0-9_]+")
+
+  assert_write_schema(cdm)
+
+  checkmate::assertLogical(computeAttrition, len = 1)
+  checkmate::assertLogical(overwrite, len = 1)
+
+  write_schema <- attr(cdm, "write_schema")
+  checkmate::assert_character(write_schema,
+                              min.chars = 1,
+                              min.len = 1,
+                              max.len = 3,
+                              null.ok = FALSE)
+
+  if ("prefix" %in% names(write_schema)) {
+    prefix <- unname(write_schema["prefix"])
+  } else {
+    prefix <- ""
+  }
+
+
+
   # Handle OHDSI cohort sets
   if ("cohortId" %in% names(cohortSet) && !("cohort_definition_id" %in% names(cohortSet))) {
     cohortSet$cohort_definition_id <- cohortSet$cohortId
@@ -206,63 +211,41 @@ generateCohortSet <- function(cdm,
   checkmate::assertCharacter(name, len = 1, min.chars = 1, pattern = "[a-z_]+")
 
   if (paste0(prefix, name) != tolower(paste0(prefix, name))) {
-    rlang::abort(glue::glue("Cohort table name `{paste0(prefix, name)}` must be lowercase!"))
+    cli::cli_abort("Cohort table name {.code {paste0(prefix, name)}} must be lowercase!")
   }
 
-  existingTables <- CDMConnector::listTables(con, write_schema)
+  # Make sure tables do not already exist
+  existingTables <- listTables(con, write_schema)
 
-  if ((paste0(prefix, name) %in% existingTables) && isFALSE(overwrite)) {
-    rlang::abort(glue::glue("The cohort table {paste0(prefix, name)} already exists.
-                            \nSpecify overwrite = TRUE to overwrite it."))
+  for (x in paste0(name, c("", "_count", "_set", "_attrition"))) {
+    if (x %in% existingTables) {
+      if (overwrite) {
+        DBI::dbRemoveTable(con, inSchema(write_schema, x, dbms = dbms(con)))
+      } else {
+        cli::cli_abort("The cohort table {paste0(prefix, name)} already exists.\nSpecify overwrite = TRUE to overwrite it.")
+      }
+    }
   }
 
-
   # Create the OHDSI-SQL for each cohort ----
 
   cohortSet$sql <- character(nrow(cohortSet))
 
   for (i in seq_len(nrow(cohortSet))) {
-    # cohortJson <- as.character(jsonlite::toJSON(cohortSet$cohort[[i]], auto_unbox = TRUE))
     cohortJson <- cohortSet$json[[i]]
     cohortExpression <- CirceR::cohortExpressionFromJson(expressionJson = cohortJson)
     cohortSql <- CirceR::buildCohortQuery(expression = cohortExpression,
                                           options = CirceR::createGenerateOptions(
                                             generateStats = computeAttrition))
     cohortSet$sql[i] <- SqlRender::render(cohortSql, warnOnMissingParameters = FALSE)
-
-    # if (dbms(con) == "snowflake") {
-      # a hack because we are using lowercase column names for cohort tables
-      # q <- function(x) DBI::dbQuoteIdentifier(con, x)
-      # r <- "DELETE FROM @target_database_schema.@target_cohort_table where cohort_definition_id = @target_cohort_id;"
-      # r2 <- glue::glue("DELETE FROM @target_database_schema.@target_cohort_table where {q('cohort_definition_id')} = @target_cohort_id;")
-      # if (!stringr::str_detect(cohortSet$sql[i], r)) rlang::abort("critical SQL replacement failed!")
-      # cohortSet$sql[i] <- stringr::str_replace(cohortSet$sql[i], r, r2)
-      #
-      # if (computeAttrition) {
-      #   r <- "delete from @results_database_schema.cohort_censor_stats where cohort_definition_id = @target_cohort_id;"
-      #   r2 <- glue::glue("delete from @results_database_schema.cohort_censor_stats where {q('cohort_definition_id')} = @target_cohort_id;")
-      #   if (!stringr::str_detect(cohortSet$sql[i], r)) rlang::abort("critical SQL replacement failed!")
-      #   cohortSet$sql[i] <- stringr::str_replace(cohortSet$sql[i], r, r2)
-      # }
-      #
-      # r <- "INSERT INTO @target_database_schema.@target_cohort_table \\(cohort_definition_id, subject_id, cohort_start_date, cohort_end_date\\)"
-      # r2 <- "INSERT INTO @target_database_schema.@target_cohort_table "
-      # if (!stringr::str_detect(cohortSet$sql[i], r)) rlang::abort("critical SQL replacement failed!")
-      # cohortSet$sql[i] <- stringr::str_replace(cohortSet$sql[i], r, r2)
-
-      # r <- "select @target_cohort_id as cohort_definition_id, person_id, start_date, end_date "
-      # r2 <- glue::glue("select @target_cohort_id as {q('cohort_definition_id')}, person_id, start_date, end_date ")
-      # if (!stringr::str_detect(cohortSet$sql[i], r)) rlang::abort("critical SQL replacement failed!")
-      # cohortSet$sql[i] <- stringr::str_replace(cohortSet$sql[i], r, r2)
-    # }
   }
-# cat(cohortSet$sql[1])
-  # Create the cohort tables ----
-  createCohortTables(con, write_schema, name, computeAttrition)
-  # Run the OHDSI-SQL ----
 
+  createCohortTables(con, write_schema, name, computeAttrition)
+
+  # Run the OHDSI-SQL ----
   cdm_schema <- attr(cdm, "cdm_schema")
   checkmate::assertCharacter(cdm_schema, max.len = 3, min.len = 1, min.chars = 1)
+
   if ("prefix" %in% names(cdm_schema)) {
     cdm_schema_sql <- glue::glue_sql_collapse(DBI::dbQuoteIdentifier(con, cdm_schema[-which(names(cdm_schema) == "prefix")]), sep = ".")
   } else {
@@ -275,102 +258,130 @@ generateCohortSet <- function(cdm,
     write_schema_sql <- paste(DBI::dbQuoteIdentifier(con, write_schema), collapse = ".")
   }
 
-  # if (!(dbms(con) %in% c("snowflake", "oracle"))) {
-    # target_cohort_table <- paste0(prefix, name)
-  # } else {
-    # target_cohort_table <- paste0(prefix, name)
+  # dropTempTableIfExists <- function(con, table) {
+  #   # used for dropping temp emulation tables
+  #   suppressMessages(
+  #     DBI::dbExecute(
+  #       con,
+  #       SqlRender::translate(
+  #         glue::glue("IF OBJECT_ID('#{table}', 'U') IS NOT NULL DROP TABLE #{table};"),
+  #         targetDialect = dbms(con))
+  #       )
+  #   )
   # }
 
-  dropTempTableIfExists <- function(con, table) {
-    suppressMessages(
-      DBI::dbExecute(
-        con,
-        SqlRender::translate(
-          glue::glue("IF OBJECT_ID('#{table}', 'U') IS NOT NULL DROP TABLE #{table};"),
-          targetDialect = dbms(con))
-        )
-    )
-  }
+  generate <- function(i) {
+    pct <- ""
+    cli::cli_progress_step("Generating cohort ({i}/{nrow(cohortSet)}{pct}) - {cohortSet$cohort_name[i]})", spinner = interactive())
 
-  for (i in seq_len(nrow(cohortSet))) {
-    # Note: you cannot quote the auxiliary cohort table names because it will end up
-    # generating sql like this: delete from "ATLAS"."RESULTS"."chrt0_inclusion"_stats where cohort_definition_id = 3
-    # which fails
     sql <- cohortSet$sql[i] %>%
       SqlRender::render(
         cdm_database_schema = cdm_schema_sql,
         vocabulary_database_schema = cdm_schema_sql,
         target_database_schema = write_schema_sql,
-        results_database_schema.cohort_inclusion        = paste0(write_schema_sql, ".", prefix, name, "_inclusion"),
-        results_database_schema.cohort_inclusion_result = paste0(write_schema_sql, ".", prefix, name, "_inclusion_result"),
-        results_database_schema.cohort_summary_stats    = paste0(write_schema_sql, ".", prefix, name, "_summary_stats"),
-        results_database_schema.cohort_censor_stats     = paste0(write_schema_sql, ".", prefix, name, "_censor_stats"),
-        results_database_schema.cohort_inclusion        = paste0(write_schema_sql, ".", prefix, name, "_inclusion"),
-        target_cohort_table = paste0(prefix, name),
+        results_database_schema.cohort_inclusion        = paste0(write_schema_sql, ".", DBI::dbQuoteIdentifier(con, paste0(prefix, name, "_inclusion"))),
+        results_database_schema.cohort_inclusion_result = paste0(write_schema_sql, ".", DBI::dbQuoteIdentifier(con, paste0(prefix, name, "_inclusion_result"))),
+        results_database_schema.cohort_summary_stats    = paste0(write_schema_sql, ".", DBI::dbQuoteIdentifier(con, paste0(prefix, name, "_summary_stats"))),
+        results_database_schema.cohort_censor_stats     = paste0(write_schema_sql, ".", DBI::dbQuoteIdentifier(con, paste0(prefix, name, "_censor_stats"))),
+        results_database_schema.cohort_inclusion        = paste0(write_schema_sql, ".", DBI::dbQuoteIdentifier(con, paste0(prefix, name, "_inclusion"))),
+        target_cohort_table = DBI::dbQuoteIdentifier(con, paste0(prefix, name)),
         target_cohort_id = cohortSet$cohort_definition_id[i],
         warnOnMissingParameters = FALSE
       )
 
-    stopifnot(length(unique(stringr::str_extract_all(sql, "@\\w+"))[[1]]) == 0)
+    if (dbms(con) == "snowflake") {
+      # we don't want to use temp emulation on snowflake. We want to use actual temp tables.
+      sql <- stringr::str_replace_all(sql, "CREATE TABLE #", "CREATE TEMPORARY TABLE ") %>%
+        stringr::str_replace_all("create table #", "create temporary table ") %>%
+        stringr::str_replace_all("#", "")
 
-    if (dbms(con) == "spark") {
-      # remove comments from SQL which are causing an issue on spark
-      # --([^\n])*?\n => match strings starting with -- followed by anything except a newline
-      sql <- stringr::str_replace_all(sql, "--([^\n])*?\n", "\n")
+      # temp tables created by circe that can be left dangling.
+      tempTablesToDrop <- c(
+        "Codesets",
+        "qualified_events",
+        "cohort_rows",
+        "Inclusion",
+        "strategy_ends",
+        "inclusion_events",
+        "included_events",
+        "final_cohort",
+        "inclusion_rules",
+        "BEST_EVENTS")
+
+      for (j in seq_along(tempTablesToDrop)) {
+        DBI::dbExecute(con, paste("drop table if exists", tempTablesToDrop[j]))
+      }
+
+      namesToQuote <- c("cohort_definition_id",
+                        "subject_id",
+                        "cohort_start_date",
+                        "cohort_end_date",
+                        "mode_id",
+                        "inclusion_rule_mask",
+                        "person_count",
+                        "rule_sequence",
+                        "gain_count",
+                        "person_total",
+                        "base_count", "final_count")
+
+      for (n in namesToQuote) {
+        sql <- stringr::str_replace_all(sql, n, DBI::dbQuoteIdentifier(con, n))
+      }
     }
 
-    tempEmulationSchema <- getOption("sqlRenderTempEmulationSchema") %||% write_schema_sql
+    # total hack workaround for circe - temp23019_chrt0_inclusion"_stats
+    quoteSymbol <- substr(as.character(DBI::dbQuoteIdentifier(con, "a")), 1, 1)
+    sql <- stringr::str_replace_all(sql,
+                             paste0("_inclusion", quoteSymbol, "_stats"),
+                             paste0("_inclusion_stats", quoteSymbol))
+
+    # if parameters exist in the sql (starting with @), stop.
+    stopifnot(length(unique(stringr::str_extract_all(sql, "@\\w+"))[[1]]) == 0)
+
+    # remove comments from SQL which are causing an issue on spark
+    # --([^\n])*?\n => match strings starting with -- followed by anything except a newline
+    sql <- stringr::str_replace_all(sql, "--([^\n])*?\n", "\n")
 
     sql <- SqlRender::translate(sql,
                                 targetDialect = CDMConnector::dbms(con),
-                                tempEmulationSchema = tempEmulationSchema) %>%
-      SqlRender::splitSql()
+                                tempEmulationSchema = "SQL ERROR")
+
+    if (stringr::str_detect(sql, "SQL ERROR")) {
+      cli::cli_abort("sqlRenderTempEmulationSchema being used for cohort generation!
+        Please open a github issue at {.url https://github.com/darwin-eu/CDMConnector/issues} with your cohort definition.")
+    }
 
     if (dbms(con) == "duckdb") {
       # hotfix for duckdb sql translation https://github.com/OHDSI/SqlRender/issues/340
       sql <- gsub("'-1 \\* (\\d+) day'", "'-\\1 day'", sql)
     }
 
-    if (!(dbms(con) %in% c("snowflake", "oracle", "bigquery"))) {
-      # TODO: issue dropping temp tables on dbms which are using tempEmulation
-      # Error: nanodbc/nanodbc.cpp:1526: 00000: Cannot perform DROP.
-      # This session does not have a current schema. Call 'USE SCHEMA', or use a qualified name.
-      # SqlRender::getTempTablePrefix()
-      dropTempTableIfExists(con, "Codesets")
-      dropTempTableIfExists(con, "qualified_events")
-      dropTempTableIfExists(con, "cohort_rows")
-      dropTempTableIfExists(con, "Inclusion")
-      dropTempTableIfExists(con, "inclusion_events")
-      dropTempTableIfExists(con, "included_events")
-      dropTempTableIfExists(con, "final_cohort")
-    } else if (dbms(con) == "bigquery") {
-      tables <- c("codesets", "qualified_events", "cohort_rows", "inclusion", "inclusion_events", "included_events", "final_cohort")
-      sql <- glue::glue("drop table if exists {write_schema}.{SqlRender::getTempTablePrefix()}{tables}")
-      purrr::walk(sql, ~DBI::dbExecute(con, .))
-    }
+    sql <- stringr::str_replace_all(sql, "\\s+", " ")
 
-    for (j in seq_along(sql)) {
-      DBI::dbExecute(con, sql[j], immediate = TRUE)
+    sql <- stringr::str_split(sql, ";")[[1]] %>%
+      stringr::str_trim() %>%
+      stringr::str_c(";") %>% # remove empty statements
+      stringr::str_subset("^;$", negate = TRUE)
+
+    for (k in seq_along(sql)) {
+      # cli::cat_rule(glue::glue("sql {k} with {nchar(sql[k])} characters."))
+      # cli::cat_line(sql[k])
+      DBI::dbExecute(con, sql[k], immediate = TRUE)
+
+      if (interactive()) {
+        pct <- ifelse(k == length(sql), "", glue::glue(" ~ {floor(100*k/length(sql))}%"))
+        cli::cli_progress_update()
+      }
     }
-    # cli::cli_progress_update(set = i)
   }
 
-  if (dbms(con) %in% c("snowflake", "oracle")) {
-    # make table lowercase
-
-    cohort_ref <- dplyr::tbl(con, inSchema(write_schema, toupper(name), dbms(con))) %>%
-      dplyr::rename_all(tolower) %>%
-      # compute_query() %>%
-      compute_query(
-        name = name,
-        temporary = FALSE,
-        schema = write_schema,
-        overwrite = TRUE
-      )
-  } else {
-    cohort_ref <- dplyr::tbl(con, inSchema(write_schema, name, dbms = dbms(con)))
+  # this loop makes cli updates look correct
+  for (i in seq_len(nrow(cohortSet))) {
+    generate(i)
   }
 
+  cohort_ref <- dplyr::tbl(con, inSchema(write_schema, name, dbms = dbms(con)))
+
   # Create attrition attribute ----
   if (computeAttrition) {
     cohort_attrition_ref <- computeAttritionTable(cdm,
@@ -382,9 +393,9 @@ generateCohortSet <- function(cdm,
   }
 
   # Create cohort_set attribute -----
-  if (paste0(name, "_set") %in% existingTables) {
-    DBI::dbRemoveTable(con, inSchema(write_schema, paste0(name, "_set"), dbms = dbms(con)))
-  }
+  # if (paste0(name, "_set") %in% existingTables) {
+  #   DBI::dbRemoveTable(con, inSchema(write_schema, paste0(name, "_set"), dbms = dbms(con)))
+  # }
 
   # overwrite not working on snowflake
   DBI::dbWriteTable(con,
@@ -419,9 +430,11 @@ generateCohortSet <- function(cdm,
     cohort_ref = cdm[[name]],
     cohort_set_ref = cohort_set_ref,
     cohort_attrition_ref = cohort_attrition_ref,
-    cohort_count_ref = cohort_count_ref)
+    cohort_count_ref = cohort_count_ref,
+    overwrite = overwrite)
 
   cli::cli_progress_done()
+
   return(cdm)
 }
 
@@ -432,7 +445,7 @@ generate_cohort_set <- function(cdm,
                                 cohort_set,
                                 name = "cohort",
                                 compute_attrition = TRUE,
-                                overwrite = FALSE) {
+                                overwrite = TRUE) {
   generateCohortSet(cdm = cdm,
                     cohortSet = cohort_set,
                     name = name,
@@ -556,7 +569,7 @@ new_generated_cohort_set <- function(cohort_ref,
                                      cohort_set_ref = NULL,
                                      cohort_attrition_ref = NULL,
                                      cohort_count_ref = NULL,
-                                     overwrite = FALSE) {
+                                     overwrite = TRUE) {
 
   if (!methods::is(cohort_ref, "tbl_sql")) {
     rlang::abort("cohort_ref must be a remote database table reference (tbl_sql)")
@@ -574,9 +587,13 @@ new_generated_cohort_set <- function(cohort_ref,
   checkmate::assertTRUE(DBI::dbIsValid(con))
   cdm <- attr(cohort_ref, "cdm_reference")
   attr(cohort_ref, "cdm_reference") <- NULL # Very important
-  if (is.null(cdm)) rlang::abort("cohort_ref must be part of a cdm!")
+  if (is.null(cdm)) {
+    rlang::abort("cohort_ref must be part of a cdm!")
+  }
   write_schema <- attr(cdm, "write_schema")
-  if (is.null(write_schema)) rlang::abort("cohort_ref must be part of a cdm with a write_schema!")
+  if (is.null(write_schema)) {
+    rlang::abort("cohort_ref must be part of a cdm with a write_schema!")
+  }
   checkmate::assert_character(write_schema, min.len = 1, max.len = 3, min.chars = 1)
   verify_write_access(con, write_schema = write_schema)
 
@@ -651,7 +668,8 @@ new_generated_cohort_set <- function(cohort_ref,
     }
 
     # primary key check
-    one <- cohort_set_ref %>%
+    if(cohort_set_ref %>% utils::head(1) %>% dplyr::tally() %>% dplyr::pull() != 0){
+      one <- cohort_set_ref %>%
       dplyr::count(.data$cohort_definition_id, name = "one") %>%
       dplyr::count(.data$one) %>%
       dplyr::pull("one")
@@ -665,6 +683,7 @@ new_generated_cohort_set <- function(cohort_ref,
       diff <- dplyr::setdiff(cohort_ids, cohort_set_ids) %>% paste(collapse = ", ")
       rlang::abort(glue::glue("cohort IDs {diff} are in the cohort table but not in the cohort_set table!"))
     }
+    }
   }
 
   # cohort_count_table ----
@@ -814,9 +833,11 @@ cohort_attrition <- cohortAttrition
 #' @export
 cohortAttrition.GeneratedCohortSet <- function(x) {
   if (is.null(attr(x, "cohort_attrition"))) {
-    NULL
+    return(invisible(NULL))
   } else {
-    dplyr::collect(attr(x, "cohort_attrition"))
+    a <- dplyr::collect(attr(x, "cohort_attrition"))
+    class(a) <- c("omop_attrition", "attrition", class(a))
+    return(a)
   }
 }
 
@@ -896,9 +917,9 @@ computeAttritionTable <- function(cdm,
 
   inclusionResultTableName <- paste0(cohortStem, "_inclusion_result")
 
-  if (dbms(attr(cdm, "dbcon")) %in% c("oracle", "snowflake")) {
-    inclusionResultTableName <- toupper(inclusionResultTableName)
-  }
+  # if (dbms(attr(cdm, "dbcon")) %in% c("oracle", "snowflake")) {
+  #   inclusionResultTableName <- toupper(inclusionResultTableName)
+  # }
 
   schema <- attr(cdm, "write_schema")
   checkmate::assertCharacter(schema, min.len = 1, max.len = 3, min.chars = 1)
