@@ -53,7 +53,7 @@ test_compute_query <- function(con, cdm_schema, write_schema) {
   expect_error({
     cdm$vocabulary  %>%
       dplyr::filter(vocabulary_id %in% c("ATC", "CPT4")) %>%
-      computeQuery(new_table_name, schema = write_schema, temporary = FALSE)},
+      computeQuery(new_table_name, schema = write_schema, temporary = FALSE, overwrite = FALSE)},
   "already exists")
 
   expect_no_error({
@@ -77,7 +77,8 @@ test_compute_query <- function(con, cdm_schema, write_schema) {
 # dbtype = "bigquery"
 for (dbtype in dbToTest) {
   test_that(glue::glue("{dbtype} - compute_query"), {
-    if (dbtype != "duckdb") skip_on_ci()
+    if (!(dbtype %in% ciTestDbs)) skip_on_ci()
+    if (dbtype != "duckdb") skip_on_cran() else skip_if_not_installed("duckdb")
     con <- get_connection(dbtype)
     cdm_schema <- get_cdm_schema(dbtype)
     write_schema <- get_write_schema(dbtype)
@@ -93,6 +94,7 @@ test_that("uniqueTableName", {
 })
 
 test_that("message does not duplicate when prefix is used", {
+  skip_if_not_installed("duckdb")
   con <- DBI::dbConnect(duckdb::duckdb(), eunomia_dir())
   cdm <- cdm_from_con(con, "main", c(prefix = "a_", schema = "main"))
   DBI::dbWriteTable(con, inSchema(attr(cdm, "write_schema"), "cars"), cars)
