@@ -1,5 +1,8 @@
 
 test_that("assertTables works with local cdms", {
+  skip_if_not_installed('arrow')
+  skip_if_not_installed("duckdb")
+  skip_on_cran()
   con <- DBI::dbConnect(duckdb::duckdb(), dbdir = eunomia_dir())
   cdm <- cdm_from_con(con, cdm_schema = "main")
 
